local mod = ...


--[[

The flock has 10 empty Attach_Minors. Each one has a component that generates a new child SpawnPoint on initialisation,
and deletes and regenerates the SpawnPoint every month. The SpawnPoint has its own component which generates the
poultry prefab that moves. SpawnPoints also have the poultry resources, and when they are collected they disappear and
remove the poultry prefab. Every month the SpawnPoint is recreated which adds a new poultry prefab to the world.

]]


-- Custom Component: Poultry Farm Attach Point -------------------------------------------------------------------------------------
local COMP_POULTRYFARM_ATTACHPOINT = {
	TypeName = "COMP_POULTRYFARM_ATTACHPOINT",
	ParentType = "COMPONENT",
	Properties = {
        {
            Name = "SpawnFlag",
            Type = "boolean",
            Default = false,
            Flags = { "SAVE_GAME" }
        },
        {
            Name = "SpawnPointId",
            Type = "string",
            Default = nil,
            Flags = { "SAVE_GAME" }
        },
        {
            Name = "PoultryType",
            Type = "string",
            Default = "CHICKEN",
            Flags = { "SAVE_GAME" }
        },
    }
}



-- Attach Point: Create Spawn Point -------------------------------------------------------------------------------------
function COMP_POULTRYFARM_ATTACHPOINT:createSpawnPoint()

    objAttach = self:getOwner()

    -- Remove existing SpawnPoint if present
    spawnId = self.SpawnPointId
    if (spawnId) then
        if string.len(spawnId) then
            objSpawn = self:getLevel():find(spawnId)
            if (objSpawn) then 
                objSpawn:destroyAllChild()
                objSpawn:destroy()
            end
        end
    end

    -- Create new SpawnPoint
    pos = objAttach:getGlobalPosition()
    objSpawn = self:getLevel():createObject("PREFAB_POULTRYFARM_SPAWNPOINT_" .. self.PoultryType, pos)
    compSpawn = objSpawn:getOrCreateComponent("COMP_POULTRYFARM_SPAWNPOINT")
    compSpawn.PoultryType = self.PoultryType
    self.SpawnPointId = objSpawn:getId()

end



-- AttachPoint: Initialise -------------------------------------------------------------------------------------
function COMP_POULTRYFARM_ATTACHPOINT:init()

    -- First time to create Spawn Point
    if not (self.SpawnFlag) then

        mod:log("COMP_POULTRYFARM_ATTACHPOINT.init ______________________________________________________________________________________")

        self:createSpawnPoint()

        -- Event to delete and regenerate every month
        local compMainGameLoop = self:getLevel():find("COMP_MAIN_GAME_LOOP")
        event.register(self, compMainGameLoop.ON_NEW_WEEK, function()
            self:createSpawnPoint()
        end)

        self.SpawnFlag = true
    end

end -- component init end



-- AttachPoint: onDestroy -------------------------------------------------------------------------------------
function COMP_POULTRYFARM_ATTACHPOINT:onDestroy(isClearingLevel)

    -- Remove any associated SpawnPoint
    if not (isClearingLevel) then
        mod:log("COMP_POULTRYFARM_ATTACHPOINT.destroy ______________________________________________________________________________________")

        if (self.SpawnPointId) then
            objSpawn = self:getLevel():find(self.SpawnPointId)
            if (objSpawn) then 
                objSpawn:destroyAllChild()
                objSpawn:destroy()
            end
        end
    
    end

end



-- Register custom Class
mod:registerClass(COMP_POULTRYFARM_ATTACHPOINT)



-- Custom Component: Poultry Farm Spawn Point -------------------------------------------------------------------------------------
local COMP_POULTRYFARM_SPAWNPOINT = {
	TypeName = "COMP_POULTRYFARM_SPAWNPOINT",
	ParentType = "COMPONENT",
	Properties = {
        {
            Name = "SpawnFlag",
            Type = "boolean",
            Default = false,
            Flags = { "SAVE_GAME" }
        },
        {
            Name = "SpawnedList",
            Type = "list<string>",
            Default = nil,
            Flags = { "SAVE_GAME" } 
        },
        {
            Name = "PoultryType",
            Type = "string",
            Default = "CHICKEN",
            Flags = { "SAVE_GAME" } 
        },
    }
}



-- Custom Component: Initialise -------------------------------------------------------------------------------------
function COMP_POULTRYFARM_SPAWNPOINT:init()

    if not (self.SpawnFlag) then

        mod:log("COMP_POULTRYFARM_SPAWNPOINT.init ______________________________________________________________________________________")

        -- Spawn a poultry prefab at the location of each SpawnPoint, record the Id of the poultry game object to delete it when the SpawnPoint is removed
        pos = self:getOwner():getGlobalPosition()
        i = math.random(1, 10)
        poultryType = self.PoultryType
        if not (poultryType) then poultryType = "CHICKEN" end
        obj = self:getLevel():createObject("PREFAB_POULTRYFARM_" .. poultryType .. i, pos)
        table.insert(self.SpawnedList, obj:getId())

        -- Enforce a radius within which the poultry can move
        compEnforcerRadius = obj:getOrCreateComponent("COMP_ENFORCE_RADIUS")
        if (compEnforcerRadius) then
            compEnforcerRadius:setEnforcer(self:getOwner())
        end

        -- Avoid respawning animals again the next time the component loads
        self.SpawnFlag = true

    end -- SpawnFlag was not set

end -- component init end



-- Custom Component: onDestroy -------------------------------------------------------------------------------------
function COMP_POULTRYFARM_SPAWNPOINT:onDestroy(isClearingLevel)

    if not (isClearingLevel) then
        mod:log("COMP_POULTRYFARM_SPAWNPOINT.destroy ______________________________________________________________________________________")
        id = self:getOwner():getId()

        -- Remove any poultry spawned from this SpawnPoint
        for i, idPair in ipairs(self.SpawnedList) do
            obj = self:getLevel():find(self.SpawnedList[i])
            obj:destroyAllChild()
            obj:destroy()
        end

    end

end



-- Register custom Class
mod:registerClass(COMP_POULTRYFARM_SPAWNPOINT)


